﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Threading;

namespace PinscapeConfigTool
{
    public partial class Form3 : Form
    {
        public Form3(DeviceInfo dev)
        {
            InitializeComponent();
            this.dev = dev;
        }

        private void Form3_Load(object sender, EventArgs e)
        {
            // start the exposure reader thread
            done = false;
            thread = new Thread(this.ExposureThread);
            thread.Start();
        }

        private void exposure_Paint(object sender, PaintEventArgs e)
        {
            // figure the number of on-screen pixels per CCD pixel
            int wid = exposure.Width;
            int ht = exposure.Height;
            double pixwid = (double)wid / (double)npix;
            int ipixwid = (int)Math.Ceiling(pixwid);

            // get the gdi handle for drawing
            Graphics g = e.Graphics;

            g.FillRectangle(Brushes.Red, new Rectangle(0, 0, wid, ht));

            // get a private copy of the current pixel array (serializing
            // access via the mutex, since the device reader thread can
            // install a new pixel array at any time)
            mutex.WaitOne();
            UInt16[] mypix = pix;
            mutex.ReleaseMutex();

            g.FillRectangle(Brushes.Green, new Rectangle(0, 0, wid, ht));

            // draw the pixels
            double x = 0;
            for (int i = 0; i < npix; ++i, x += pixwid)
            {
                byte lum = (byte)(mypix[i] >> 8);
                Brush b = new SolidBrush(Color.FromArgb(lum, lum, lum));
                g.FillRectangle(b, new Rectangle((int)Math.Round(x), 0, ipixwid, ht));
            }
        }

        private void Form3_FormClosed(object sender, FormClosedEventArgs e)
        {
            done = true;
        }

        private void ExposureThread()
        {
            // create a clone of the device record to get a private handle
            // for the thread
            DeviceInfo tdev = new DeviceInfo(dev);

            while (!done)
            {
                // send a request for the next pixel frame list
                tdev.SpecialRequest(3);

                // read reports until we see an exposure report at index 0;
                // if we don't see one in a reasonable amount of time, abort
                // this round and send a new request - we might have missed
                // the one we asked for
                for (int i = 0; i < 20 && !done; ++i)
                {
                    // read a report
                    byte[] buf = tdev.ReadUSB();

                    // if it's an exposure report at index 0, start reading
                    if (buf != null && buf[1] == 0x00 && buf[2] == 0x80)
                    {
                        // create a local array for reading the results
                        UInt16[] newpix = new UInt16[npix];

                        // continue until we've read all pixels
                        for (int j = 0 ; j < 1000 && !done ; ++j)
                        {
                            // only consider reports that are in the special exposure report format
                            if (buf != null && (buf[2] & 0xF8) == 0x80)
                            {
                                // figure the starting index of the current batch
                                int idx = (((int)buf[2] & 0x7) << 8) | (int)buf[1];

                                // store this batch
                                for (int k = 3; k < buf.Length && idx < npix ; k += 2)
                                    newpix[idx++] = (UInt16)(((UInt16)buf[k + 1] << 8) | (UInt16)buf[k]);

                                // if this is the last batch, we're done
                                if (idx >= npix)
                                    break;
                            }

                            // read the next report
                            buf = tdev.ReadUSB();
                        }

                        // copy the new pixel list to the drawing copy
                        mutex.WaitOne();
                        pix = newpix;
                        mutex.ReleaseMutex();

                        // figure the statistics
                        UInt16 pixMin = 0xFFFF, pixMax = 0x0000;
                        int numSat = 0, numZero = 0;
                        for (int j = 0; j < npix; ++j)
                        {
                            UInt16 p = pix[j];
                            if (p == 0)
                                ++numZero;
                            if (p == 0xffff)
                                ++numSat;
                            if (p < pixMin)
                                pixMin = p;
                            if (p > pixMax)
                                pixMax = p;
                        }

                        // update the stats label
                        txtInfo.Text = String.Format("Min: {0}, Max: {1}, %saturated: {2}, %zero: {3}",
                            pixMin, pixMax, (int)Math.Round(numSat * 100.0 / npix), 
                            (int)Math.Round(numZero * 100.0 / npix));

                        // invalidate the visual display so we redraw it with the new data
                        exposure.Invalidate();

                        // done reading this pixel list - go back for the next one
                        break;
                    }
                }
            }
        }

        private DeviceInfo dev;

        private bool done;

        private const int npix = 160;
        private UInt16[] pix = new UInt16[npix];

        private Mutex mutex = new Mutex();
        private Thread thread;

        private void btnClose_Click(object sender, EventArgs e)
        {
            ActiveForm.Close();
        }
    }
}
